#<profile4xi profile = "02" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "forkawarethreadlock4xi.py">
#<abstract4xi abstract = "provides the class ForkAwareThreadLock4Xi"/>
#<refer4xi reference = "util.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------


# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'ForkAwareThreadLock4Xi' ]
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass

                
try:                 # most-common ones
    import itertools
    import weakref
    import atexit         
except:
    pass
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from register_after_fork4xi import register_after_fork4xi
from print4xi import print4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi
from time4xi import time4xi
#"
#/>

# =================================================================================================
#
#<class4xi class = "ForkAwareThreadLock4Xi" base = "object" child = "None"
#  synopsis = "used for multiprocessing by managers"
#  usage = "
#     managers.py:
#         _mutex = util.ForkAwareThreadLock4Xi()
#     synchronize.py:
#          _mutex = util.ForkAwareThreadLock4Xi()
#
#     forkawarethreadlock4xi.py
#         register_after_fork4xi(self, ForkAwareThreadLock4Xi.__init__)
#"
#>
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# =================================================================================================
class ForkAwareThreadLock4Xi(object):
    '''
    is used for multiprocessing by managers  
    '''

    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi exposure = "
    _exposed_ = ('sense', 'reason', 'ace')          
    #"
    #/>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-ForkAwareThreadLock4Xi" usage = "initializes ForkAwareThreadLock4Xi">                           
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __init__(self):
        '''
        initializes ForkAwareThreadLock4Xi
        '''
        # -----------------------------------------------------------------------------------------
        # (Step 1/6)
        # -----------------------------------------------------------------------------------------
        class2print = "ForkAwareThreadLock4Xi"
        method2print = "__init__"

        message2print = "{(Step 1/6a) started" + "; init-of-ForkAwareThreadLock4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   

        message2print = "{(Step 1/6b) borning" + "; forkawarethreadlock2xi-of-ForkAwareThreadLock4Xi: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           

        message2print = "{(Step 1/6c) inheriting" + "; object-inheriting-by-forkawarethreadlock2xi-of-ForkAwareThreadLock4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                        
        # -----------------------------------------------------------------------------------------
        # (Step 2/6)
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2/6a) calling" + "; threading-Lock-calling-by-init-of-ForkAwareThreadLock4Xi: " + str(threading.Lock) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
                        
        try:
            self._lock = threading.Lock()
       
            message2print = "{(Step 2/6b) set" + "; lock-of-forkawarethreadlock2xi-set-by-init-of-ForkAwareThreadLock4Xi to: " + str(self._lock)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        except:
            message2print = "{(Step 2/6c) hit" + "; threading-Lock-hit-by-init-of-ForkAwareThreadLock4Xi: " + " *** " + str(threading.Lock)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
        
            raise        
                             

        # -----------------------------------------------------------------------------------------
        # (Step 3/6)
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 3/6a) using" + "; lock-acquire-of-forkawarethreadlock2xi-using-by-init-of-ForkAwareThreadLock4Xi: " + str(self._lock.acquire)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                    
        try:                                   
            self.acquire = self._lock.acquire
       
            message2print = "{(Step 3/6b) set" + "; acquire-of-forkawarethreadlock2xi-set-by-init-of-ForkAwareThreadLock4Xi to: " + str(self.acquire)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        except:
            message2print = "{(Step 3/6c) hit" + "; lock-acquire-of-forkawarethreadlock2xi-hit-by-init-of-ForkAwareThreadLock4Xi: " + " *** " + str(self._lock.acquire)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
        
            raise                      
                

        # -----------------------------------------------------------------------------------------
        # (Step 4/6)
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 4/6a) using" + "; lock-release-of-forkawarethreadlock2xi-using-by-init-of-ForkAwareThreadLock4Xi: " + str(self._lock.release)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        try:                                         
            self.release = self._lock.release

       
            message2print = "{(Step 4/6b) set" + "; release-of-forkawarethreadlock2xi-set-by-init-of-ForkAwareThreadLock4Xi to: " + str(self.release)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        except:
            message2print = "{(Step 4/6c) hit" + "; lock-release-of-forkawarethreadlock2xi-hit-by-init-of-ForkAwareThreadLock4Xi: " + " *** " + str(self._lock.release)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
        
            raise              
                 

        # -----------------------------------------------------------------------------------------
        # (Step 5/6)
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 5/6a) passing" + "; forkawarethreadlock2xi-passing-to-register-after-fork4xi-by-init-of-ForkAwareThreadLock4Xi: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                
        message2print = "{(Step 5/6b) passing" + "; ForkAwareThreadLock4Xi-init-passing-to-register-after-fork4xi-by-init-of-ForkAwareThreadLock4Xi: " + str(ForkAwareThreadLock4Xi.__init__) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                      
        try:
        
            register_after_fork4xi(self, ForkAwareThreadLock4Xi.__init__)
        
            message2print = "{(Step 5/6c) performed" + "; register_after_fork4xi-performed-by-init-of-ForkAwareThreadLock4Xi: " + str(register_after_fork4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
        except:
            message2print = "{(Step 5/6d) hit" + "; register-after-fork4xi-hit-by-init-of-ForkAwareThreadLock4Xi: " + " *** " + str(register_after_fork4xi)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
               
            raise    
                

        # -----------------------------------------------------------------------------------------
        # (Step 6/6)
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 6/6a) inherited" + "; object-inherited-by-forkawarethreadlock2xi-of-ForkAwareThreadLock4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                        
        message2print = "{(Step 6/6b) activated" + "; forkawarethreadlock2xi-of-ForkAwareThreadLock4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  

                   
        message2print = "{(Step 6/6c) returning" + "; init-of-ForkAwareThreadLock4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                        
     
                                      
     #</methodize4xi>
                                           
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-ForkAwareThreadLock4Xi" usage = "senses ForkAwareThreadLock4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses ForkAwareThreadLock4Xi class objects
        '''
        
        
        class2print = 'ForkAwareThreadLock4Xi'
        method2print = 'sense'
        
        message2print = "{started" + "; sense-of-ForkAwareThreadLock4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        message2print = "{sensing" + "; ForkAwareThreadLock4Xi-sensing-by-sense-of-ForkAwareThreadLock4Xi: " + str(ForkAwareThreadLock4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                     
   
                        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{got" + "; kb2xi-got-by-sense-of-ForkAwareThreadLock4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
    
        message2print = "{sensed" + "; ForkAwareThreadLock4Xi-sensed-by-sense-of-ForkAwareThreadLock4Xi: " + str(ForkAwareThreadLock4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
         

        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        message2print = "{returning" + "; kb2xi-returning-by-sense-of-ForkAwareThreadLock4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                              
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-ForkAwareThreadLock4Xi" usage = "reasons ForkAwareThreadLock4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons ForkAwareThreadLock4Xi class objects
        '''
             
        
        class2print = 'ForkAwareThreadLock4Xi'
        method2print = 'reason'

        message2print = "{started" + "; reason-of-ForkAwareThreadLock4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)     
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; ForkAwareThreadLock4Xi-reasoning-by-reason-of-ForkAwareThreadLock4Xi: " + str(ForkAwareThreadLock4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
                
                
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        message2print = "{got" + "; kb4xi-got-by-reason-of-ForkAwareThreadLock4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        
        message2print = "{reasoned" + "; ForkAwareThreadLock4Xi-reasoned-by-reason-of-ForkAwareThreadLock4Xi: " + str(ForkAwareThreadLock4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         

        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb4xi-returning-by-reason-of-ForkAwareThreadLock4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                         
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-ForkAwareThreadLock4Xi" usage = "aces ForkAwareThreadLock4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces ForkAwareThreadLock4Xi class objects 
        '''
            
        
        
        class2print = 'ForkAwareThreadLock4Xi'
        method2print = 'ace'
        
        message2print = "{started" + "; ace-of-ForkAwareThreadLock4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; ForkAwareThreadLock4Xi-acing-by-ace-of-ForkAwareThreadLock4Xi: " + str(ForkAwareThreadLock4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{got" + "; kb8xi-got-by-ace-of-ForkAwareThreadLock4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
          
        message2print = "{aced" + "; ForkAwareThreadLock4Xi-aced-by-ace-of-ForkAwareThreadLock4Xi: " + str(ForkAwareThreadLock4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb8xi-returning-by-ace-of-ForkAwareThreadLock4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                            
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
#</class4xi>

#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of forkawarethreadlock4xi.py                              
# ------------------------------------------------------------------------------------------------- 
#"
#/>          

